@@ -15,10 +15,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *      MA 02110-1301, USA.
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /* HTML Characters plugin (Inserts HTML character entities like '&amp;') */
@@ -561,7 +560,7 @@ static void sc_fill_store(GtkTreeStore *store)
 		if (chars[i][1] == NULL)
 		{	/* add a category */
 			gtk_tree_store_append(store, &iter, NULL);
-			gtk_tree_store_set(store, &iter, COLUMN_CHARACTER, chars[i][0], -1);
+			gtk_tree_store_set(store, &iter, COLUMN_CHARACTER, _(chars[i][0]), -1);
 			if (parent_iter != NULL) gtk_tree_iter_free(parent_iter);
 			parent_iter = gtk_tree_iter_copy(&iter);
 		}
@@ -588,7 +587,7 @@ static gboolean sc_insert(GtkTreeModel *model, GtkTreeIter *iter)
 		gint pos = sci_get_current_position(doc->editor->sci);
 
 		gtk_tree_model_get(model, iter, COLUMN_HTML_NAME, &str, -1);
-		if (NZV(str))
+		if (!EMPTY(str))
 		{
 			sci_insert_text(doc->editor->sci, pos, str);
 			g_free(str);
@@ -737,7 +736,7 @@ static void init_configuration(void)
 void plugin_init(GeanyData *data)
 {
 	GtkWidget *menu_item;
-	const gchar *menu_text = _("_Insert Special HTML Characters");
+	const gchar *menu_text = _("_Insert Special HTML Characters...");
 
 	/* First we catch the configuration and initialize them */
 	init_configuration();
