@@ -69,7 +69,7 @@ void sci_set_line_numbers(ScintillaObject *sci, gboolean set, gint extra_width)
 
 void sci_set_mark_long_lines(ScintillaObject *sci, gint type, gint column, const gchar *colour)
 {
-	glong colour_val = utils_strtod(colour, NULL, TRUE); /* Scintilla uses a "long" value */
+	glong colour_val = utils_parse_color_to_bgr(colour); /* Scintilla uses a "long" value */
 
 	if (column == 0)
 		type = 2;
@@ -375,6 +375,12 @@ gint sci_get_col_from_position(ScintillaObject *sci, gint position)
 }
 
 
+gint sci_get_position_from_col(ScintillaObject *sci, gint line, gint col)
+{
+	return (gint) SSM(sci, SCI_FINDCOLUMN, line, col);
+}
+
+
 /** Gets the position for the start of @a line.
  * @param sci Scintilla widget.
  * @param line Line.
@@ -1247,12 +1253,6 @@ gint sci_get_position_after(ScintillaObject *sci, gint start)
 }
 
 
-void sci_lines_split(ScintillaObject *sci, gint pixelWidth)
-{
-	SSM(sci, SCI_LINESSPLIT, (uptr_t) pixelWidth, 0);
-}
-
-
 void sci_lines_join(ScintillaObject *sci)
 {
 	SSM(sci, SCI_LINESJOIN, 0, 0);
@@ -1264,12 +1264,27 @@ gint sci_text_width(ScintillaObject *sci, gint styleNumber, const gchar *text)
 	return (gint) SSM(sci, SCI_TEXTWIDTH, (uptr_t) styleNumber, (sptr_t) text);
 }
 
+
 void sci_move_selected_lines_down(ScintillaObject *sci)
 {
 	SSM(sci, SCI_MOVESELECTEDLINESDOWN, 0, 0);
 }
 
+
 void sci_move_selected_lines_up(ScintillaObject *sci)
 {
 	SSM(sci, SCI_MOVESELECTEDLINESUP, 0, 0);
 }
+
+
+gint sci_word_start_position(ScintillaObject *sci, gint position, gboolean onlyWordCharacters)
+{
+	return SSM(sci, SCI_WORDSTARTPOSITION, position, onlyWordCharacters);
+}
+
+
+gint sci_word_end_position(ScintillaObject *sci, gint position, gboolean onlyWordCharacters)
+{
+	return SSM(sci, SCI_WORDENDPOSITION, position, onlyWordCharacters);
+}
+
