@@ -30,7 +30,7 @@ gchar *win32_show_file_dialog(GtkWindow *parent, const gchar *title, const gchar
 gboolean win32_show_document_open_dialog(GtkWindow *parent, const gchar *title, const gchar *initial_dir);
 
 gchar *win32_show_document_save_as_dialog(GtkWindow *parent, const gchar *title,
-										  const gchar *initial_file);
+										  GeanyDocument *doc);
 
 void win32_show_font_dialog(void);
 
@@ -52,8 +52,6 @@ void win32_init_debug_code(void);
 
 void win32_set_working_directory(const gchar *dir);
 
-gchar *win32_get_hostname(void);
-
 gboolean win32_get_exit_status(GPid child_pid);
 
 gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags flags,
