@@ -91,7 +91,7 @@ void tm_workspace_free(gpointer workspace)
 	}
 }
 
-const TMWorkspace *tm_get_workspace()
+const TMWorkspace *tm_get_workspace(void)
 {
 	if (NULL == theWorkspace)
 		tm_create_workspace();
@@ -146,7 +146,7 @@ gboolean tm_workspace_load_global_tags(const char *tags_file, gint mode)
 	guchar buf[BUFSIZ];
 	FILE *fp;
 	TMTag *tag;
-	gboolean format_pipe = FALSE;
+	TMFileFormat format = TM_FILE_FORMAT_TAGMANAGER;
 
 	if (NULL == theWorkspace)
 		return FALSE;
@@ -163,24 +163,33 @@ gboolean tm_workspace_load_global_tags(const char *tags_file, gint mode)
 	else
 	{	/* We read the first line for the format specification. */
 		if (buf[0] == '#' && strstr((gchar*) buf, "format=pipe") != NULL)
-			format_pipe = TRUE;
+			format = TM_FILE_FORMAT_PIPE;
 		else if (buf[0] == '#' && strstr((gchar*) buf, "format=tagmanager") != NULL)
-			format_pipe = FALSE;
+			format = TM_FILE_FORMAT_TAGMANAGER;
+		else if (buf[0] == '#' && strstr((gchar*) buf, "format=ctags") != NULL)
+			format = TM_FILE_FORMAT_CTAGS;
+		else if (strncmp((gchar*) buf, "!_TAG_", 6) == 0)
+			format = TM_FILE_FORMAT_CTAGS;
 		else
 		{	/* We didn't find a valid format specification, so we try to auto-detect the format
 			 * by counting the pipe characters on the first line and asumme pipe format when
 			 * we find more than one pipe on the line. */
-			guint i, pipe_cnt = 0;
+			guint i, pipe_cnt = 0, tab_cnt = 0;
 			for (i = 0; i < BUFSIZ && buf[i] != '\0' && pipe_cnt < 2; i++)
 			{
 				if (buf[i] == '|')
 					pipe_cnt++;
+				else if (buf[i] == '\t')
+					tab_cnt++;
 			}
-			format_pipe = (pipe_cnt > 1);
+			if (pipe_cnt > 1)
+				format = TM_FILE_FORMAT_PIPE;
+			else if (tab_cnt > 1)
+				format = TM_FILE_FORMAT_CTAGS;
 		}
 		rewind(fp); /* reset the file pointer, to start reading again from the beginning */
 	}
-	while (NULL != (tag = tm_tag_new_from_file(NULL, fp, mode, format_pipe)))
+	while (NULL != (tag = tm_tag_new_from_file(NULL, fp, mode, format)))
 		g_ptr_array_add(theWorkspace->global_tags, tag);
 	fclose(fp);
 
@@ -198,7 +207,7 @@ static guint tm_file_inode_hash(gconstpointer key)
 #ifdef TM_DEBUG
 		g_message ("Hash for '%s' is '%d'\n", filename, file_stat.st_ino);
 #endif
-		return g_direct_hash (GUINT_TO_POINTER (file_stat.st_ino));
+		return g_direct_hash ((gpointer)(gulong)file_stat.st_ino);
 	} else {
 		return 0;
 	}
@@ -585,8 +594,9 @@ const GPtrArray *tm_workspace_find(const char *name, int type, TMTagAttrType *at
 	else
 		tags = g_ptr_array_new();
 
-	matches[0] = tm_tags_find(theWorkspace->work_object.tags_array, name, partial, &tagCount[0]);
-	matches[1] = tm_tags_find(theWorkspace->global_tags, name, partial, &tagCount[1]);
+	matches[0] = tm_tags_find(theWorkspace->work_object.tags_array, name, partial, TRUE,
+					&tagCount[0]);
+	matches[1] = tm_tags_find(theWorkspace->global_tags, name, partial, TRUE, &tagCount[1]);
 
 	/* file tags */
 	if (matches[0] && *matches[0])
@@ -681,7 +691,7 @@ fill_find_tags_array (GPtrArray *dst, const GPtrArray *src,
 	if ((!src) || (!dst) || (!name) || (!*name))
 		return 0;
 
-	match = tm_tags_find (src, name, partial, &count);
+	match = tm_tags_find (src, name, partial, TRUE, &count);
 	if (count && match && *match)
 	{
 		for (tagIter = 0; tagIter < count; ++tagIter)
